from zope.interface import Interface
from zope.interface.common.mapping import IEnumerableMapping
from zope import schema

from collective.geo.mapwidget import GeoMapwidgetMessageFactory as _


class IMaps(IEnumerableMapping):
    """A mapping form mapids to IMapWidgets

    looked up as ((view, request, context), IMaps)
    """


class IMapLayers(IEnumerableMapping):
    """
    A mapping of IMapLayer instances

    specific for IMapWidget... looked up as
      ((view, request, context, mapwidget), name)
    """

    js = schema.TextLine(
        title=_(u"Javascript to configure layers."),
        description=_("Returns some js-code to set up available layers."),
        required=True)


class IMapWidget(Interface):
    """
    Provides configuration options for a specific map widget.
    """
    mapid = schema.TextLine(
        title=_(u"Map id"),
        description=_(u"Used to identify the map in the dom-tree and to "\
                      u"lookup an IMapWidget component if necessary."),
        default=u"default-cgmap",
        required=True)

    klass = schema.TextLine(
        title=_(u"Class attribute"),
        description=_(u"The html element class attribute."),
        default=u"widget-cgmap",
        required=True)

    style = schema.TextLine(
        title=_(u"Style attribute"),
        description=_(u"The html element style attribute."),
        required=False)

    js = schema.Text(
        title=_("Javascript extras"),
        description=_(u"Additional Javascript code inserted after the map"
                      u" widget."),
        required=False)

    layers = schema.Object(
        title=_('Layers'),
        description=_('A mapping from layerids to ILayers'),
        schema=IMapLayers)

    usedefault = schema.Bool(
        title=_(u"Enable default layers."),
        description=_(u"If set to true, the default IMapLayers implementation"
                      u" adds all enabled default layers from the geo settings"
                      u" tool."),
        default=True,
        required=False)

    def addClass(klass):
        '''
        add klass to self.klass
        '''


class IMapView(Interface):
    """
    A view implementing this interface provides configurable
    map widgets.
    """
    # TODO: is this the right field for an IMapView or should it be
    #       mapfields here?
    mapwidgets = schema.Object(
        title=_('Map Widgets'),
        description=_('A mapping from mapids to IMapWidgets'),
        schema=IMaps)


class IMapLayer(Interface):
    """
    A pluggable interface making it easier to configure layers.
    """

    jsfactory = schema.Text(
        title=_(u"Javascript factory"),
        description=_(
            u"Javascript code which returns a new instance of this "
            u"layer and does not expect any parameters"
        ),
        required=True)


class IDefaultMapLayers(Interface):
    """Utility to provide a list of default map layers
    """


class IGeoCoder(Interface):
    """Adapter for geocoding feature
    """

    def retrieve(address=None, google_api=None, language=None):
        """Retrieve coordinates by an address

        :param address: a string representing an address
            to be converted in coordinates
        :type address: string
        :param google_api: google api token
        :type google_api: string
        :returns: a sequence of coordinates representing a point
        :rtype: tuple
        """



from zope.component import getUtility, queryAdapter

from plone.registry.interfaces import IRegistry
from collective.geo.settings.interfaces import IGeoCustomFeatureStyle
from collective.geo.settings.interfaces import IGeoFeatureStyle
from ..utils import get_feature_styles

MAP_STYLE_FIELDS = ['map_width', 'map_height']


class GeoFeatureStyleView(object):
    """Supporting Geo Feature Style settings used for maps
    """

    def __init__(self, context, request):
        """Initialise our feature style view with relevant map styles.

        If custom styles are specified for our context and they are enabled,
        then use them for styling our map. Otherwise, default to using the
        site-wide styles.
        """

        self.context = context
        self.request = request
        self.styles = get_feature_styles(context)

    def __getattribute__(self, name):
        """Proxy attribute access to our local styles.

        If something has requested one of the fields in our custom styles
        then we get the property from there.  Otherwise, provide access
        normally using the parent method.
        """
        if name in MAP_STYLE_FIELDS:
            return self.styles.get(name, None)
        else:
            return super(GeoFeatureStyleView, self).__getattribute__(name)



from plone.indexer.decorator import indexer
from collective.geo.settings.interfaces import IGeoFeatureStyle
from .utils import get_feature_styles
from zope.interface import Interface


@indexer(IGeoFeatureStyle)
def collective_geo_styles(context):
    styles = get_feature_styles(context)
    return styles



from z3c.form.interfaces import IWidget
from collective.geo.mapwidget.interfaces import IMapView
from collective.geo.mapwidget.browser.widget import MapWidget
from collective.geo.mapwidget.maplayers import MapLayer


class ShapeMapWidget(MapWidget):

    mapid = 'geoshapemap'

    @property
    def js(self):
        wkt_field_id = None
        if IWidget.providedBy(self.view):
            wkt_field_id = self.view.id
        elif getattr(self.view, 'widgets', None):
            wkt_field = self.view.widgets.get('wkt')
            if wkt_field:
                wkt_field_id = wkt_field.id

        if wkt_field_id:
            return """
(function ($) {
    $(window).load(function() {
        // collective geo edit map
        $('#%s').collectivegeo(
            'add_edit_layer',
            '%s'
        );
        $('#%s').collectivegeo('add_geocoder');
    });
}(jQuery));
""" % (self.mapid, wkt_field_id)



import urllib
import unittest
from decimal import Decimal

from zope.component import getUtility
from plone.registry.interfaces import IRegistry

from collective.geo.settings.interfaces import IGeoSettings

from ..testing import CGEO_MAPWIDGET_INTEGRATION


class TestSetupHTTP(unittest.TestCase):
    layer = CGEO_MAPWIDGET_INTEGRATION

    _protocol = 'http'

    def setUp(self):
        self.portal = self.layer['portal']
        self._old_request_url = self.portal.REQUEST.getURL()
        self.portal.REQUEST.setServerURL(self._protocol, 'nohost')
        self.settings = self.portal.restrictedTraverse('@@geosettings-view')
        self.geosettings = getUtility(IRegistry).forInterface(IGeoSettings)

    def beforeTearDown(self):
        url_parts = urllib.splittype(self._old_request_url)
        self.portal.REQUEST.setServerURL(*url_parts)

    def test_property_zoom(self):
        self.assertEquals(self.settings.zoom, Decimal("10.0"))

    def test_property_map_center(self):
        self.assertEquals(
            self.settings.map_center,
            (Decimal("0.00"), Decimal("0.0"))
        )

    def test_property_googleapi(self):
        key = u'ABQIAAAAaKes6QWqobpCx2AOamo-shTwM0brOpm-'\
              'All5BF6PoaKBxRWWERSUWbHs4SIAMkeC1KV98E2EdJKuJw'
        self.assertEquals(self.settings.googleapi, key)

    def test_property_jsgooglemaps(self):
        # when a layer is google_map we should include external javascript
        self.geosettings.default_layers = [u'google_map']
        self.assertEquals(
            self.settings.google_maps_js,
            '%s://maps.google.com/maps/api/js?v=3.2&sensor=false&key=%s' %
            (self._protocol, self.settings.googleapi)
        )

        self.geosettings.default_layers = [u'osm']
        self.assertFalse(self.settings.googlemaps)

    def test_property_jsbingmaps(self):
        # when a layer is bing_map we should include external javascript
        self.geosettings.default_layers = [u'bing_map']
        self.assertEquals(
            self.settings.bing_maps_js,
            '%s://dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6' %
            self._protocol
        )

        self.geosettings.default_layers = [u'osm']
        self.assertFalse(self.settings.bingmaps)

    def test_geosettings_properties(self):
        self.assertEqual(
            self.settings.zoom,
            10
        )

        self.assertEqual(
            self.settings.map_center,
            (Decimal('0.0'), Decimal('0.0'))
        )

        self.assertEqual(
            self.settings.imgpath,
            "string:${portal_url}/img/"
        )


class TestSetupHTTPS(TestSetupHTTP):
    _protocol = 'https'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSetupHTTP))
    suite.addTest(unittest.makeSuite(TestSetupHTTPS))
    return suite



from Acquisition import aq_inner
from zope.component import getUtility
from zope.component import getMultiAdapter
from Products.CMFCore.Expression import Expression, getExprContext

from plone.memoize import instance, view
from plone.registry.interfaces import IRegistry

from collective.geo.settings.interfaces import IGeoSettings
from collective.geo.mapwidget import utils


_BINGURL = '%s://dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6'
_GOOGLEURL = '%s://maps.google.com/maps/api/js?v=3.2&sensor=false&key=%s'


class GeoSettingsView(object):
    """Geo Settings macros
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.layer_protocol = utils.getProtocolFromRequest(self.request)
        self.geosettings = getUtility(IRegistry).forInterface(IGeoSettings)

    @property
    def default_layers(self):
        return self.geosettings.default_layers

    @property
    def zoom(self):
        return  self.geosettings.zoom

    @property
    def map_center(self):
        return self.geosettings.longitude, self.geosettings.latitude

    @property
    def imgpath(self):
        return  self.geosettings.imgpath

    @property
    def googlemaps(self):
        """Include google javascript only whether c.geo
        uses a google map layer and it isn't already
        included in the page
        """
        if not self.request.get('googlemaps_js'):
            for layer_id in self.default_layers:
                if layer_id.startswith('google'):
                    self.request.set('googlemaps_js', True)
                    return True
        return False

    @property
    def googleapi(self):
        return  self.geosettings.googleapi

    @property
    def google_maps_js(self):
        """return google maps 3 api javascript url
        """
        return _GOOGLEURL % (self.layer_protocol, self.googleapi)

    @property
    def bingapi(self):
        return  self.geosettings.bingapi

    @property
    def bingmaps(self):
        """Include bing javascript only whether c.geo
        uses a bing map layer and it isn't already
        included in the page
        """
        if not self.request.get('bingmaps_js'):
            for layer_id in self.default_layers:
                if layer_id.startswith('bing'):
                    self.request.set('bingmaps_js', True)
                    return True
        return False

    @property
    def bing_maps_js(self):
        """return Bing maps javascript url
        """
        return _BINGURL % self.layer_protocol

    @property
    def location(self):
        try:
            location = self.context.getLocation()
        except AttributeError:
            return u''
        if isinstance(location, str):
            return location.decode('utf8')

    @property
    def localize(self):
        """ Returns True if the widget should be localized.
        """
        if self.request.get('openlayers_js') or self.language == 'en':
            return False

        self.request.set('openlayers_js', True)
        return self.language in self.language_files

    @property
    @view.memoize
    def language(self):
        """ Return the languagecode of the current context.
        """
        portal_state = getMultiAdapter(
            (self.context, self.request),
            name="plone_portal_state")
        lang = aq_inner(self.context).Language() or \
            portal_state.default_language()

        return lang.lower()

    @property
    @instance.memoize
    def language_files(self):
        """Get, cache and return a list of openlayers language files.
        """
        return utils.list_language_files()

    @property
    def language_file(self, language=None):
        """ Returns the path to the openlayers language file of the
        given or the current language.
        """
        return self.language_files[language or self.language]

    @property
    def geo_setting_js(self):
        # Default configuration has been moved on mapwidgets
        # TODO: remove this property
        return None



try:
    import json
except ImportError:
    import simplejson as json
from zope.component import getUtility, getMultiAdapter
from Products.Five.browser import BrowserView
from geopy.exc import GeocoderQueryError

from ..interfaces import IGeoCoder


class GeoCoderView(BrowserView):
    """A simple view which provides a json output from geopy query.
    """

    def __init__(self, context, request):
        super(GeoCoderView, self).__init__(context, request)
        self.geocoder = getUtility(IGeoCoder)

    def __call__(self, address=None, google_api=None):
        context = self.context.aq_inner
        portal_state = getMultiAdapter((context, self.request),
                                       name=u'plone_portal_state')
        language = portal_state.language()

        try:
            locations = self.geocoder.retrieve(address, google_api, language)
        except GeocoderQueryError:
            return 'null'

        if not locations:
            return 'null'

        return json.dumps(
            [(loc.address, (loc.latitude, loc.longitude)) for loc in locations]
        )



from zope.component import getMultiAdapter
from zope.publisher.browser import BrowserView

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

INLINE_STYLES = {'width': 'map_width',
                 'height': 'map_height'}


class CollectiveGeoMacros(BrowserView):
    template = ViewPageTemplateFile('collectivegeo_macros.pt')

    def __getitem__(self, key):
        return self.template.macros[key]

    def map_inline_css(self):
        """Return inline CSS for our map according to style settings.
        """
        inline_css = ''
        geofeaturestyle = getMultiAdapter(
            (self.context, self.request),
            name='geofeaturestyle-view'
        )
        for style in INLINE_STYLES:
            value = getattr(geofeaturestyle, INLINE_STYLES[style], None)
            if value:
                inline_css += "%s:%s;" % (style, value)

        return inline_css or None



from zope.component.hooks import getSite
from zope.component import getGlobalSiteManager
from Products.CMFCore.utils import getToolByName

default_profile = 'profile-collective.geo.mapwidget:default'


def upgrade_to_20(context):
    """This upgrade step fixes javascript registry and static resources
    """
    context.runImportStepFromProfile(default_profile, 'jsregistry')
    context.runImportStepFromProfile(default_profile, 'controlpanel')

    # install collective.z3cform.mapwidget and colorpicker
    qi = getToolByName(context, 'portal_quickinstaller')

    if not qi.isProductInstalled('collective.z3cform.colorpicker'):
        qi.installProduct('collective.z3cform.colorpicker')

    if not qi.isProductInstalled('collective.z3cform.mapwidget'):
        qi.installProduct('collective.z3cform.mapwidget')



from zope.interface import Interface
from zope.component import getAdapters
from collective.geo.settings.vocabularies import baseVocabulary
from collective.geo.mapwidget.interfaces import IMapLayer


class maplayersVocab(baseVocabulary):

    @property
    def terms(self):
        all_layers = getAdapters((Interface,
                        Interface, Interface, Interface), IMapLayer)
        for layer_id, layer in all_layers:
            yield((layer_id, layer.Title))



"""
This module contains IMapLayer implementations for commonly available
base maps. These layers can be configured in the geo-settings control panel
or may be re-used in manually configured map-widgets.
"""
from zope.interface import implements
from zope.component import getUtility, queryMultiAdapter
from zope.component import getMultiAdapter

from plone.registry.interfaces import IRegistry
from plone.memoize.instance import memoizedproperty

from collective.geo.settings.interfaces import IGeoSettings
from collective.geo.mapwidget.interfaces import IMapLayer
from collective.geo.mapwidget.interfaces import IDefaultMapLayers

from collective.geo.mapwidget import GeoMapwidgetMessageFactory as _
from collective.geo.mapwidget import utils


class MapLayer(object):
    '''
    An empty IMapLayer implementation, useful as base class.

    MapLayers are named components specific for
    (view, request, context, widget).
    '''

    implements(IMapLayer)
    Title = u""
    name = u""
    # we need a property to evaluate if the layer map is based on google
    # or bing maps to include a external javascrpt
    type = 'base'

    def __init__(self, view=None, request=None, context=None, widget=None):
        self.view = view
        self.request = request
        self.protocol = utils.getProtocolFromRequest(self.request)
        self.context = context
        self.widget = widget

    @memoizedproperty
    def jsfactory(self):
        try:
            template = getMultiAdapter(
                (self.context, self.request),
                name=str('%s-layer' % self.name)
            )
        except AttributeError:
            return u""
        return template() % dict(title=self.Title, protocol=self.protocol)


class OSMMapLayer(MapLayer):
    name = u"osm"
    Title = _(u"OpenStreetMap")


class BingMapLayer(MapLayer):

    @memoizedproperty
    def jsfactory(self):
        settings = getUtility(IRegistry).forInterface(IGeoSettings)
        api_key = settings.bingapi
        try:
            template = getMultiAdapter(
                (self.context, self.request),
                name=str('%s-layer' % self.name)
            )
        except AttributeError:
            return u""
        return template() % {
            'title': self.Title,
            'protocol': self.protocol,
            'apiKey': api_key
        }


class BingStreetMapLayer(BingMapLayer):
    name = u"bing_map"
    Title = _(u"Bing")
    #type = 'bing'


class BingRoadsMapLayer(BingMapLayer):
    name = u"bing_rod"
    Title = _(u"Bing Roads")
    #type = 'bing'


class BingAerialMapLayer(BingMapLayer):
    name = u"bing_aer"
    Title = _(u"Bing Aerial")
    #type = 'bing'


class BingHybridMapLayer(BingMapLayer):
    name = u"bing_hyb"
    Title = _(u"Bing Hybrid")
    #type = 'bing'


class GoogleStreetMapLayer(MapLayer):
    name = u"google_map"
    Title = _(u"Google")
    type = 'google'


class GoogleSatelliteMapLayer(MapLayer):
    name = u"google_sat"
    Title = _(u"Satellite (Google)")
    type = 'google'


class GoogleHybridMapLayer(MapLayer):
    name = u"google_hyb"
    Title = _(u"Hybrid (Google)")
    type = 'google'


class GoogleTerrainMapLayer(MapLayer):
    name = u"google_ter"
    Title = _(u"Terrain (Google)")
    type = 'google'


class ShapeDisplayLayer(MapLayer):

    name = 'shapedisplay'

    @property
    def Title(self):
        return u'%s' % self.context.Title().decode('utf-8')

    @memoizedproperty
    def jsfactory(self):
        try:
            template = getMultiAdapter(
                (self.context, self.request),
                name=str('%s-layer' % self.name)
            )
        except AttributeError:
            return u""

        return template() % {
            'title': self.Title,
            'coords': self.widget.coords()
        }


class DefaultMapLayers(object):
    """Utility to store default map layers
    """

    implements(IDefaultMapLayers)

    @property
    def geo_settings(self):
        return getUtility(IRegistry).forInterface(IGeoSettings)

    @property
    def default_layers(self):
        return (u'osm', )

    def layers(self, view, request, context, widget):
        default_layers = self.geo_settings.default_layers
        if not default_layers:
            default_layers = self.default_layers

        layers = []
        for layerid in default_layers:
            layer = queryMultiAdapter(
                (view, request, context, widget),
                IMapLayer,
                name=layerid
            )
            if layer:
                layers.append(layer)

        return layers



import unittest
try:
    import json
except ImportError:
    import simplejson as json

from zope.component import getUtility

from plone.testing.z2 import Browser
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import setRoles

from geopy.exc import GeocoderQueryError

from ..utils import GeoCoderUtility
from ..interfaces import IGeoCoder
from ..testing import CGEO_MAPWIDGET_INTEGRATION


test_params = [
    {
        'address': "Torino Italy",
        'output': [
            (
                u'Turin, Italy',
                (45.070562099999997, 7.6866186000000001)
            )
        ]
    },

    {
        'address': "Serravalle Italy",
        'output': [
            (
                u'Serravalle di Chienti Macerata, Italy',
                (43.075758700000002, 12.957291700000001)
            ),
            (
                u'46030 Serravalle a Po Mantua, Italy',
                (45.071769699999997, 11.0986653)
            ),
            (
                u'Serravalle, 50019 Sesto Fiorentino Florence, Italy',
                (43.847528799999999, 11.2683242)
            ),
            (
                u'Serravalle, 12026 Piasco Cuneo, Italy',
                (44.5675697, 7.4256900000000003)
            ),
            (
                u'Serravalle, 06046 Norcia Perugia, Italy',
                (42.785488399999998, 13.022334499999999)
            ),
            (
                u'Serravalle, 54023 Filattiera Massa-Carrara, Italy',
                (44.367425699999998, 9.9383029000000001)
            ),
            (
                u'Serravalle, Berra Ferrara, Italy',
                (44.967833300000002, 12.044703699999999)
            ),
            (
                u'Serravalle, Asti, Italy',
                (44.947478799999999, 8.1465417999999996)
            ),
            (
                u'Serravalle, Bibbiena Arezzo, Italy',
                (43.7736485, 11.8429064)
            ),
            (
                u'Serravalle, 38061 Ala Trento, Italy',
                (45.811786499999997, 11.0141562)
            )
        ]
    }
]


class DummyGeoCoderResult(object):

    def __init__(self, data):
        self.address = data[0]
        self.latitude = data[1][0]
        self.longitude = data[1][1]


class DummyGeoCoder(GeoCoderUtility):

    def retrieve(self, address=None, google_api=None,
                 language=None):  # pylint: disable=W0613
        for item in test_params:
            if address == item['address']:
                return [DummyGeoCoderResult(res) for res in item['output']]
        raise GeocoderQueryError


class TestGeocoder(unittest.TestCase):
    layer = CGEO_MAPWIDGET_INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.oid = self.portal.invokeFactory('Document', 'doc')
        setRoles(self.portal, TEST_USER_ID, ['Member'])

        self.obj = self.portal[self.oid]
        self.geo = getUtility(IGeoCoder)

    def test_geocoder_base(self):
        for item in test_params:
            locations = self.geo.retrieve(item['address'])
            self.assertEquals(
                [(loc.address, (loc.latitude, loc.longitude))
                    for loc in locations],
                item['output']
            )

    def test_geocoder_error(self):
        self.assertRaises(GeocoderQueryError,
                          self.geo.retrieve,
                          "not existent place aklhj asaas")

    def test_geocoder_view(self):
        browser = Browser(self.layer['app'])
        browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,)
        )

        for item in test_params:
            obj_url = (
                "%s/@@geocoderview?address=%s" %
                (self.portal.absolute_url(), item['address'])
            )
            browser.open(obj_url)
            view_contents = json.loads(browser.contents)
            i = 0
            for place, (lat, lon) in view_contents:
                test_place, (test_lat, test_lon) = item['output'][i]
                self.assertEquals(test_place, place)
                self.assertEquals(test_lat, lat)
                self.assertEquals(test_lon, lon)
                i += 1


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestGeocoder))
    return suite



import unittest
import doctest

from zope.component import getGlobalSiteManager
from zope.interface import Interface

from plone.testing import layered

from ..interfaces import IMapLayer
from ..testing import CGEO_MAPWIDGET_FUNCTIONAL


def tearDown(test):  # pylint: disable=W0613
    # unregister image Maplayer
    gsm = getGlobalSiteManager()
    adpt = gsm.queryMultiAdapter(
        (Interface, Interface, Interface, Interface),
        IMapLayer,
        name='image')
    if adpt:
        gsm.unregisterAdapter(
            adpt.__class__,
            (Interface, Interface, Interface, Interface),
            IMapLayer,
            name='image'
        )


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([

        layered(doctest.DocFileSuite(
            'README.txt',
            package='collective.geo.mapwidget',
            tearDown=tearDown,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | \
                doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
            ),
            layer=CGEO_MAPWIDGET_FUNCTIONAL
        ),

        layered(doctest.DocFileSuite(
            'controlpanel.txt',
            package='collective.geo.mapwidget.browser',
            tearDown=tearDown,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | \
                doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
            ),
            layer=CGEO_MAPWIDGET_FUNCTIONAL
        ),
    ])
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




from zope.interface import implements
from zope.component import getMultiAdapter, getUtility
from zope.publisher.interfaces.browser import IBrowserView

from Products.Five import BrowserView

from collective.geo.mapwidget.interfaces import (
    IMaps, IMapWidget, IMapLayer,
    IMapLayers, IMapView,
    IDefaultMapLayers
)


class MapView(BrowserView):
    '''
    Helper view to look up mapwidgets for current view and context.
    '''
    # TODO: shall this be a IMapView or the view itself?
    implements(IMapView)

    def mapwidgets(self):
        # if IBrowserView.providedBy(self.context):
        return getMultiAdapter(
            (self.context, self.request, self.context.context),
            IMaps
        )
        # return []


class MapWidgets(list):
    '''IMaps adapter which initialises IMapWidgets
    for current view ad context.
    '''
# TODO: ensure all these methods are available
#    def __getitem__(key):
#         """Get a value for a key
#
#         A KeyError is raised if there is no value for the key.
#         """
#
#     def get(key, default=None):
#         """Get a value for a key
#
#         The default is returned if there is no value for the key.
#         """
#
#     def __contains__(key):
#         """Tell if a key exists in the mapping."""
#
# def keys():
#         """Return the keys of the mapping object.
#         """
#
#
#     def __iter__():
#         """Return an iterator for the keys of the mapping object.
#         """
#
#     def values():
#         """Return the values of the mapping object.
#         """
#
#     def items():
#         """Return the items of the mapping object.
#         """
#
#     def __len__():
#         """Return the number of items.
#         """

    implements(IMaps)

    def __init__(self, view, request, context):
        self.keys = {}
        self.view = view
        self.request = request
        self.context = context
        mapfields = getattr(view, 'mapfields', None)
        if mapfields:
            for mapid in mapfields:
                if IMapWidget.providedBy(mapid):
                    # is already a MapWidget, just take it
                    self.__append(mapid.mapid, mapid)
                elif isinstance(mapid, basestring):
                    # is only a name... lookup the widget
                    self.__append(
                        mapid,
                        getMultiAdapter(
                            (self.view, self.request, self.context),
                            IMapWidget,
                            name=mapid
                        )
                    )
                else:
                    raise ValueError(
                        "Can't create IMapWidget for %s" % repr(mapid)
                    )
        else:
            # there are no mapfields let's look up the default widget
            self.__append(
                'default-cgmap',
                getMultiAdapter(
                    (self.view, self.request, self.context),
                    IMapWidget,
                    name='default-cgmap'))

    def __append(self, key, value):
        self.keys[key] = value
        self.append(value)

    def __getitem__(self, key):
        if isinstance(key, basestring):
            return self.keys[key]
        return super(MapWidgets, self).__getitem__(key)


class MapWidget(object):
    '''The default IMapWidget, which also can serve as handy base class.
    '''

    implements(IMapWidget)

    mapid = 'default-cgmap'
    klass = 'widget-cgmap'
    js = "// default"
    _layers = []

    def __init__(self, view, request, context):
        self.view = view
        self.request = request
        self.context = context

    @property
    def layers(self):
        return getMultiAdapter(
            (self.view, self.request, self.context, self),
            IMapLayers
        )

    def addClass(self, klass):
        if not self.klass:
            self.klass = unicode(klass)
        else:
            # Make sure items are not repeated.
            parts = self.klass.split() + [unicode(klass)]
            self.klass = u' '.join(frozenset(parts))

    def map_defaults(self):
        settings = getMultiAdapter(
            (self.context, self.request), name='geosettings-view'
        )
        lon, lat = settings.map_center
        pstate = getMultiAdapter(
            (self.context, self.request), name='plone_portal_state'
        )
        portal_url = pstate.portal_url()

        # Image path for changing OpenLayers default images.
        # TODO: check if settings are overriden for this context
        try:
            expr = Expression(str(settings.imgpath))
            imgpath = expr(getExprContext(self.context))
        except:
            imgpath = ''

        return {
            'longitude': lon,
            'latitude': lat,
            'zoom': settings.zoom,
            'imgpath': imgpath,
            'geocoderurl': "%s/geocoderview" % portal_url
        }


        # set default configuration
        # ret = ["cgmap.state = {'default': " \
        #     "{lon: %(lon)7f, lat: %(lat)7f, zoom: %(zoom)d }};" % state]
        # # go through all maps in request and extract their state
        # # to update map_state
        # for mapid in self.request.get('cgmap_state_mapids', '').split():
        #     map_state = self.request.get('cgmap_state.%s' % mapid)
        #     state = {'mapid': mapid}
        #     for param in ('lon', 'lat', 'zoom',
        #                   'activebaselayer', 'activelayers'):
        #         val = map_state.get(param, None)
        #         state[param] = (val is not None) and ("'%s'" % val) or \
        #                                                             'undefined'
        #     ret.append("cgmap.state['%(mapid)s'] = " \
        #             "{lon: %(lon)s, lat: %(lat)s, zoom: %(zoom)s, " \
        #             "activebaselayer: %(activebaselayer)s, activelayers: " \
        #             "%(activelayers)s };" % state)

        # import pdb; pdb.set_trace( )


class MapLayers(dict):
    '''
    The default IMapLayers implementation.

    Checks geo settings tool for enabled layers and adds them
    if enabled (widget.usedefault).

    TODO: this impl is too tigly copled with the default MapWigdet
          implementation.
          esp.: it should not look for widget._layers attribute.
    '''

    implements(IMapLayers)

    def __init__(self, view, request, context, widget):
        self.view = view
        self.request = request
        self.context = context
        self.widget = widget

    def layers(self):
        # shall I use getAllAdapters instead of getNamed?
        layers = []
        useDefaultLayers = getattr(self.widget, 'usedefault', True)
        if useDefaultLayers:
            default_layers = getUtility(IDefaultMapLayers)
            layers.extend(
                default_layers.layers(
                    self.view, self.request,
                    self.context, self.widget
                )
            )

        maplayers = getattr(self.widget, '_layers', None)
        if maplayers:
            for layerid in maplayers:
                if IMapLayer.providedBy(layerid):
                    layers.append(layerid)
                elif isinstance(layerid, basestring):
                    layers.append(
                        getMultiAdapter(
                            (
                                self.view, self.request,
                                self.context, self.widget
                            ),
                            IMapLayer,
                            name=layerid
                        )
                    )
                else:
                    raise ValueError(
                        "Can't create IMapLayer for %s" % repr(layerid))
        return layers

    @property
    def js(self):
        layers = self.layers()
        return """
$(window).bind('mapload', function (evt, widget) {
    widget.addLayers([
        %(layers)s
    ], '%(mapid)s');
});

""" % {
            'layers': ",\n".join([l.jsfactory for l in layers]),
            'mapid': self.widget.mapid
        }



# -*- coding: utf-8 -*-
import urllib
from geopy import geocoders
from geopy.exc import GeocoderQueryError

from zope.interface import implements
from zope.schema import getFields
from zope.component import getUtility
from Products.CMFCore import DirectoryView

from plone import api
from plone.registry.interfaces import IRegistry
from collective.geo.settings.interfaces import IGeoFeatureStyle

from .interfaces import IGeoCoder


def get_feature_styles(context):
    fields = [i for i in getFields(IGeoFeatureStyle)]
    manager = IGeoFeatureStyle(context, None)
    use_custom_styles = getattr(manager, 'use_custom_styles', False)
    if not use_custom_styles:
        registry = getUtility(IRegistry)
        manager = registry.forInterface(IGeoFeatureStyle)
    styles = {
        'use_custom_styles': use_custom_styles
    }
    for name in fields:
        styles[name] = getattr(manager, name, None)

    return styles


def getProtocolFromRequest(request):
    """Determine which protocol layers should use for this request.

    Layers should, where possible, match the request protocol to avoid
    client browsers reporting errors to users.
    """
    server_url = request and request.get('SERVER_URL', '') or None
    return server_url and urllib.splittype(server_url)[0] or 'http'


def list_language_files():
    """ Returns a dictionary of available Openlayers translations.

    Keys are the language codes (lowercase, 2-4 in length).
    Values are the urls to the language files.
    """

    path = 'collective.geo.openlayers:skins/geo_openlayers/lang'
    urlbase = 'lang/'
    files = dict()

    info = DirectoryView._dirreg.getDirectoryInfo(path)

    if info:
        for key in info.getContents(DirectoryView._dirreg)[0].keys():
            files[key.replace('.js', '').lower()] = urlbase + key

    return files


def get_geocoder():
    return GeoCoderUtility()


class GeoCoderUtility(object):
    """Wrapper class for geopy
    """
    implements(IGeoCoder)

    def retrieve(self, address=None, google_api=None, language=None):
        if google_api is None:
            google_api = api.portal.get_registry_record(
                'collective.geo.settings.interfaces.IGeoSettings.googleapi',
                default=None,
            )
        self.geocoder = geocoders.GoogleV3(api_key=google_api)

        if not address:
            raise GeocoderQueryError
        return self.geocoder.geocode(address, exactly_one=False,
                                     language=language)



import unittest

from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory

from zope.component import getUtility
from zope.component import queryUtility
from zope.component import getAdapters

from zope.publisher.browser import TestRequest

from ..interfaces import IDefaultMapLayers, IMapLayer
from ..testing import CGEO_MAPWIDGET_INTEGRATION


class TestDefaultMapLayers(unittest.TestCase):
    layer = CGEO_MAPWIDGET_INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']

    def test_utility(self):
        self.failUnless(queryUtility(IDefaultMapLayers))

    def _get_utility(self):
        utility = getUtility(IDefaultMapLayers)
        default_layers = [i[0] for i in self._get_registered_layers()]
        utility.geo_settings.default_layers = default_layers
        return utility

    def _get_registered_layers(self):
        return getAdapters((Interface,
                            Interface,
                            Interface,
                            Interface), IMapLayer)

    def test_layers_vocabulary(self):
        layer_ids = [i[0] for i in self._get_registered_layers()]
        vocab_util = queryUtility(IVocabularyFactory,
                        name='maplayersVocab')

        self.failUnless(vocab_util)

        vocab = vocab_util(self.portal)
        for item in vocab:
            self.assertIn(
                item.value,
                layer_ids,
                "term %s not in map layers" % item.value)

    def test_default_settings(self):
        layers = [u'osm', u'google_ter', u'google_hyb',
                  u'google_sat', u'google_map']
        utility = getUtility(IDefaultMapLayers)

        self.assertEquals(utility.geo_settings.default_layers, layers)

    def test_default_layers(self):
        layer_ids = [i[0] for i in self._get_registered_layers()]
        layers_utility = self._get_utility()
        layers = layers_utility.layers(None, None, None, None)
        self.assertEquals(len(layers), len(layer_ids))

    def test_select_layers_by_name(self):
        layer_ids = [i[0] for i in self._get_registered_layers()]
        layers_utility = self._get_utility()
        layers_utility.geo_settings.default_layers = layer_ids
        layers = layers_utility.layers(None, None, None, None)

        self.assertEquals(len(layers), len(layer_ids))

        for layer in layers:
            self.assertIn(
                layer.name,
                layer_ids,
                "%s not in map layers" % layer.name
            )

    def test_layer_protocols(self):
        """Test layers know what protocol is being used - HTTP or HTTPS."""
        layers_utility = self._get_utility()

        for protocol in ['https', 'http']:
            request = TestRequest(environ={'SERVER_URL':
                                           '%s://nohost' % protocol})

            self.assertEquals(request['SERVER_URL'], '%s://nohost' % protocol)

            layers = layers_utility.layers(None, request, None, None)
            for layer in layers:
                self.assertEquals(layer.protocol, protocol)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestDefaultMapLayers))
    return suite



from setuptools import setup, find_packages
import os

version = '2.5'

setup(name='collective.geo.mapwidget',
      version=version,
      description="collective.geo mapwidget",
      long_description=open(
          "README.rst").read() + "\n" + open(
              os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
          "Framework :: Plone",
          "Framework :: Plone :: 4.3",
          "Intended Audience :: End Users/Desktop",
          "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
          "Topic :: Internet",
          "Topic :: Internet :: WWW/HTTP",
          "Topic :: Scientific/Engineering :: GIS",
          "Topic :: Office/Business :: Groupware",
          "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      keywords='Zope Plone GIS KML Google Maps Bing OpenLayers',
      author='Giorgio Borelli',
      author_email='giorgio@giorgioborelli.it',
      url='https://github.com/collective/collective.geo.mapwidget',
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective', 'collective.geo'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'BeautifulSoup',
          'geopy>=0.98',
          'Products.CMFCore',
          'plone.app.z3cform',
          'collective.geo.openlayers >= 3.1',
          'collective.geo.settings >= 3.1',
          'collective.z3cform.colorpicker >= 1.2',
      ],
      extras_require={
          'test': [
              'plone.app.testing',
          ]
      },
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from Acquisition import aq_inner

from zope.schema import Choice
from zope.schema.interfaces import IVocabularyFactory
from zope.interface import implements
from zope.component import queryUtility
from zope.event import notify

from z3c.form import field, form, subform, button
from z3c.form.interfaces import IFormLayer

from plone.z3cform import z2
from plone.z3cform.fieldsets import extensible, group

from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.utils import getToolByName

from collective.z3cform.colorpicker.colorpickeralpha import (
    ColorpickerAlphaFieldWidget
)

from collective.geo.settings.interfaces import IGeoSettings, IGeoFeatureStyle
from collective.geo.settings.events import GeoSettingsEvent

from collective.geo.mapwidget.interfaces import IMapView
from collective.geo.mapwidget.browser.widget import MapWidget
from collective.geo.mapwidget.maplayers import MapLayer

from collective.geo.mapwidget import GeoMapwidgetMessageFactory as _


def back_to_controlpanel(context):
    portal_url = context.restrictedTraverse('plone_portal_state').portal_url()
    return dict(url='%s/plone_control_panel' % portal_url)


class GeopointForm(subform.EditSubForm):
    heading = _(u"Default map position")
    contents_top = _(
        u"Set the centre point and the zoom level of the map "
        u"used when the system cannot use any other point to center the map "
        u"(e.g. when displaying the map "
        u"to georeference an item for the first time)")
    template = ViewPageTemplateFile('geopointform.pt')
    implements(IMapView)
    fields = field.Fields(IGeoSettings).select('longitude', 'latitude',
                                               'zoom')
    mapfields = ['geosettings-cgmap']

    def update(self):
        self.updateWidgets()

    def applyChanges(self, data):
        content = self.getContent()
        return form.applyChanges(self, content, data)


class GeoStylesGroup(group.Group):

    label = _(u"Style")
    description = _(u"Set default styles for geographical shapes")

    @property
    def fields(self):
        fields = field.Fields(IGeoFeatureStyle).select('marker_image')
        fields += field.Fields(IGeoSettings).select('imgpath')
        fields += field.Fields(IGeoFeatureStyle).select(
            'marker_image_size',
            'map_width',
            'linecolor',
            'map_height',
            'polygoncolor',
            'display_properties',
            'linewidth',
        )
        fields['linecolor'].widgetFactory = ColorpickerAlphaFieldWidget
        fields['polygoncolor'].widgetFactory = ColorpickerAlphaFieldWidget
        return fields


class GeoAdvancedConfGroup(group.Group):

    label = _(u"Advanced")
    description = _(u"Advanced configurations")

    fields = field.Fields(IGeoSettings).select('map_viewlet_managers')

    def updateWidgets(self):
        super(GeoAdvancedConfGroup, self).updateWidgets()
        widgets = ('googleapi', 'bingapi')
        for w in self.widgets:
            if w in widgets:
                self.widgets[w].size = 80
                self.widgets[w].update()


def control_panel_fields():
    form_fields = field.Fields(IGeoSettings).select(
        'default_layers', 'bingapi', 'googleapi')
    form_fields += field.Fields(IGeoFeatureStyle).select(
        'map_viewlet_position')
    default_layer_field = form_fields['default_layers']
    default_layer_field.field.value_type = Choice(
        title=_(u"Layers"),
        source="maplayersVocab"
    )
    return form_fields


class GeoControlpanelForm(extensible.ExtensibleForm, form.EditForm):
    template = ViewPageTemplateFile('form-with-subforms.pt')
    form.extends(form.EditForm, ignoreButtons=True)

    default_fieldset_label = _(u"Base settings")

    heading = _(u'Configure Collective Geo Settings')
    fields = control_panel_fields()
    groups = (GeoStylesGroup, GeoAdvancedConfGroup)

    level = 1

    @property
    def css_class(self):
        return "subform openlayers-level-%s" % self.level

    def __init__(self, context, request):
        super(GeoControlpanelForm, self).__init__(context, request)
        _subform = GeopointForm(self.context, self.request, self)
        self.ptool = getToolByName(self.context, 'plone_utils')
        _subform.level = self.level + 1

        self.subforms = [_subform, ]

    def update(self):
        # update subforms first, else the values won't
        # be available in button handler
        for _subform in self.subforms:
            _subform.update()
        super(GeoControlpanelForm, self).update()

    @button.buttonAndHandler(_(u'Apply'), name='apply')
    def handle_apply(self, action):
        subdata, suberrors = self.subforms[0].extractData()
        data, errors = self.extractData()

        if errors or suberrors:
            self.status = self.formErrorsMessage
            return

        changes = self.applyChanges(data)
        coordinate_changes = self.subforms[0].applyChanges(subdata)
        if changes or coordinate_changes:
            self.status = self.successMessage
            notify(GeoSettingsEvent(self.context, data))
        else:
            self.status = self.noChangesMessage

        self.ptool.addPortalMessage(self.status, 'info')
        self.request.response.redirect(self.back_link)

    @button.buttonAndHandler(_(u'Cancel'), name='cancel')
    def handle_cancel(self, action):
        self.ptool.addPortalMessage(self.noChangesMessage, 'info')
        self.request.response.redirect(self.back_link)

    @property
    def back_link(self):
        return back_to_controlpanel(self.context)['url']


# TODO: Extend plone.z3cform.FormWrapper
class GeoControlpanel(BrowserView):
    __call__ = ViewPageTemplateFile('controlpanel.pt')

    label = _(u'Geo Settings')
    description = _(u"Collective Geo Default Settings")
    request_layer = IFormLayer
    form = GeoControlpanelForm

    # NOTE: init code taken from plone.z3cform FormWrapper...
    #       maybe extending FormWrapper would be an option?
    def __init__(self, context, request):
        super(GeoControlpanel, self).__init__(context, request)
        if self.form is not None:
            self.form_instance = self.form(
                aq_inner(self.context),
                self.request
            )
            self.form_instance.__name__ = self.__name__

    @property
    def back_link(self):
        return back_to_controlpanel(self.context)['url']

    def contents(self):
        z2.switch_on(self)
        self.form_instance.update()
        return self.form_instance.render()

    def update(self):
        # see:
        # Module plone.app.z3cform.kss.validation, line 47, in validate_input
        # AttributeError: 'GeoControlpanel' object has no attribute 'update'
        self.form_instance.update()
        self.prefix = self.form_instance.prefix
        self.widgets = self.form_instance.widgets
        self.groups = self.form_instance.groups

    def extractData(self):
        """Stupid fix for plone.app.z3cform v. 0.5.0 - kss validation
        """
        return self.form_instance.extractData()


class ControlPanelMapWidget(MapWidget):

    mapid = 'geosettings-cgmap'

    @property
    def js(self):
        return None



# -*- coding: utf-8 -*-
from plone.app.testing import PloneWithPackageLayer
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting

import collective.geo.mapwidget


CGEO_MAPWIDGET = PloneWithPackageLayer(
    zcml_package=collective.geo.mapwidget,
    zcml_filename='testing.zcml',
    gs_profile_id='collective.geo.mapwidget:default',
    name="CGEO_MAPWIDGET")

CGEO_MAPWIDGET_INTEGRATION = IntegrationTesting(
    bases=(CGEO_MAPWIDGET, ),
    name="CGEO_MAPWIDGET_INTEGRATION")

CGEO_MAPWIDGET_FUNCTIONAL = FunctionalTesting(
    bases=(CGEO_MAPWIDGET, ),
    name="CGEO_MAPWIDGET_FUNCTIONAL")
